#!/usr/local/perl -dw
#
# DISAMBIGUATION PROGRAM USING CONCEPTUAL DENSITY
# CONCEPTUAL DENSITY MODULE
#
# Author: Eneko Agirre Bengoa
#         Ixa Taldea NLP group
#         Computer Languages and Systems Department
#         Basque Country University
#
# Copyright (c) Eneko Agirre 1998
#
# No redistribution without prior permission. Published work containing
# results derived from use of this software must contain an appropriate
# acknowledgement.
#
# Lisp file: wn-w.lisp
#

# use Math::BigFloat ;;
# Problems with this module: does not have ** operation

package Dens ;

use Exporter () ;
@ISA = qw(Exporter) ;
@EXPORT = qw(dens_node dens_compute_mu) ;

# use strict ;


#
# external parameters:
# Tree::$ALPHA        # alpha in density function
# Tree::$BETA         # beta in density function
#
sub dens_node ($$$;$$) {
  my($descendants,$mu,$marks,$alpha,$beta) = @_ ;

  return 0 if $marks == 0 ;

  $alpha = 1 if !defined($alpha) ;
  $beta = 0 if !defined($beta) ;

  # compute summatory: $i=0 -> $sum=1 ;
  $sum = 1 ;
  for ($i = 1; $i < $marks; $i++) {
    $sum += ($mu + $beta) ** ($i ** $alpha) ;
  }
  return( $sum / $descendants );
}

#
# Formula for computing average branching factor (number of hyponyms or mu)
# Finds x for: A = x^0 + x^1 + ... +x^N (N = $heigth - 1, A = $desc) 
#
# There is an improved formula using integrals in wn-w.lisp

sub dens_compute_mu ($$) {
  my($desc,$h) = @_ ;
  my($prec,$r,$firstr,$s,$e,$fdesc,$fh) = (0.0000001,0,0,0) ;

#  $prec = Math::BigFloat->new("0.000000001") ;
#  $r    = Math::BigFloat->new(0) ;
#  $s    = Math::BigFloat->new(0) ;
#  $e    = Math::BigFloat->new(0) ;
#  $fdesc= Math::BigFloat->new($desc) ;
#  $fh   = Math::BigFloat->new($h) ;

  return 0 if $h == 0 ;
  return 1 if $h == 1 || $desc == 1 ;

  $h = $h - 1 ;
  if ($desc < (1 + $h)) {
    $r = 1 - 1/$desc ; }
  else {
    $r = $desc ** (1 / $h) ; }
  
  $firstr = $r ;
  for (;;) {
    $s = $r ** $h ;
    $e = ($r * ($s - $desc) + $desc -1) / ($s * ($h + 1) - $desc) ;
    $r = $r - $e ;
    return $firstr if ($r == 0) or ($e == 0) or ($s == 0) ; # if ($r eq "NaN") or ($e eq "NaN") or ($s eq "NaN")  ;
    return $r      if ( abs($e/$r) < $prec) ;
  }
}

(1)
