#!/usr/local/perl -dw
#
# DISAMBIGUATION PROGRAM USING CONCEPTUAL DENSITY
#
# Author: Eneko Agirre Bengoa
#         Ixa Taldea NLP group
#         Computer Languages and Systems Department
#         Basque Country University
#
# Copyright (c) Eneko Agirre 1998
#
# No redistribution without prior permission. Published work containing
# results derived from use of this software must contain an appropriate
# acknowledgement.
#
# interface:
#    &wsddens_list ;
#


package WsdDens ;

#
# MAIN PARAMETERS
#
use vars qw($ALPHA $BETA $PARTOF $LOCALMU $GLOBALMU $WORDS) ;  # these are global variables 

BEGIN {                #default values, can be overiden at runtime
  $ALPHA    = 0.2 ;    # alpha in density function
  $BETA     = 0;       # beta in density function
  $PARTOF   = "";      # use meronyms (".pof") or not ("")
  $LOCALMU  = 0;       # use global mu or local mu (mu = average number of hyponyms)
  $GLOBALMU = 4.41;    # in WN 1.4 
  $WORDS    = "";      # count marks using words (".words") or concepts ("")
}


use Tree ; 

use Exporter () ;
@ISA = qw(Exporter) ;
@EXPORT = qw(wsddens_list wsddens_list2 ALPHA BETA PARTOF) ;

use strict ;


#
# disambiguate list of words of same pos
# input:
#   list of words with optional concept coding
#
# output:
#   $result_ref{$word}{$c} = $weight
#    
#
# LISP: wn-main.lisp::min-distance-english-noun

sub wsddens_list (@) {
  my(@list) = @_ ;
  my($word) ;
  my($selected_concept, @selected_concepts,$result_ref,$c) ;

  tree_init_counter() ;
  tree_init("n",@list) ;
  tree_delete_cycles() ;
  tree_compute_dens() ;
  for (;;) {
    $selected_concept = tree_select_max_concept() ;
    last if ! $selected_concept ;
    push @selected_concepts, $selected_concept ;
    tree_mark_disambiguated($selected_concept) ;
  }
  tree_explain_disambiguated(@selected_concepts) if $main::DEBUG;
  $result_ref = tree_return_disambiguated() ;
  dump_result($result_ref) if $main::DEBUG;
  return($result_ref) ;
}

#
# disambiguate list of words of same pos BUT it doesn't delete anything,
# that is, it returns all possible selected concepts in order
#
# input:
#   list of words with optional concept coding
#
# LISP: wn-main.lisp::min-distance-english-noun

sub wsddens_list2 (@) {
  my(@list) = @_ ;
  my($word) ;
  my($selected_concept, @selected_concepts,$result_ref,$c) ;

  tree_init_counter() ;
  tree_init("n",@list) ;
  tree_delete_cycles() ;
  tree_compute_dens() ;
  for (;;) {
    $selected_concept = tree_select_next_max_concept($selected_concept) ;
    last if ! $selected_concept ;
    push @selected_concepts, $selected_concept ;
  }
  tree_explain_disambiguated(@selected_concepts) if $main::DEBUG;
  $result_ref = tree_return_disambiguated() ;
  dump_result($result_ref) if $main::DEBUG;
  return($result_ref) ;
}


sub dump_result ($) {
  my($resref) = @_ ;
  my($w,$c) ;

  foreach $w (keys %{ $resref }) {
    foreach $c (keys %{ ${ $resref }{$w} }) {
      print "$w: $c ${ $resref }{$w}{$c}\n" ;}}
}

(1)
